from __future__ import unicode_literals

import csv
import six

from driverng import MessageV2, Worklist
from driverng.splitted import String


class SigmoidaResult(object):
    FIELDS = [
        String("well"),
        String("sample"),
        String("target"),
        String("cq"),
        String("result"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for elt, field in enumerate(fields):
            setattr(self, field.name, line[elt])


class SigmoidaResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: SigmoidaResult}

    def add(self, type_message=MessageV2.RESULT):
        obj = self._MESSAGE_TYPE_MAP[type_message]()
        self.append(obj)
        return obj

    def parse(self, data):
        buf = six.StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        for line in reader:
            if not len(line):
                continue
            if len(line) > 4:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "sigmoida", "label": "SIGMOIDA Resultats CSV"}

    def parse(self, data):
        obj = SigmoidaResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            if m.well == "Well":
                continue
            sample = worklist.get_or_create(
                sample_id=m.sample, rack=m.well, type="patient"
            )
            sample.results.add(code="{}Cq".format(m.target), value=m.cq)
            sample.results.add(code="{}Result".format(m.target), value=m.result)
        return worklist
